/***************************************************************************
These C++ classes are copyright 1990, by William Herrera.
All those who put this code or its derivatives in a commercial product MUST
mention this copyright in their documentation for users of the products in
which this code or its derivative classes are used.  Otherwise, this code
may be freely distributed and freely used for any purpose.
***************************************************************************/

// file modem.cpp class definitions for modem class.

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "modem.hpp"

const char inits[] = "AT &Q0 &Q3 M1 S7=60 S11=55 V1 S0=6";
const char tonedials[] = "ATDT";
const char pulsedials[] = "ATDP";
const char answers[] = "ATA";
const char hangups[] = "ATH0";
const char resets[] = "ATZ";
const char retrns[] = "\r\n";
const char escapes[] = "+++";	
const char defaultcfgs[] = "modem.cfg";

modem::modem(int portnum, long speed, parity_t p,
						 int sbits, int dbits, boolean trans)
 : SerialPort(portnum, speed, p, sbits, dbits, trans)
{
	init = strdup(inits);
	tonedial = strdup(tonedials);
	pulsedial = strdup(pulsedials);
	answer = strdup(answers);
	hangup = strdup(hangups);
	reset = strdup(resets);
	retrn = strdup(retrns);
	escape = strdup(escapes);
}

modem::~modem()
{
	free(init);
	free(tonedial);
	free(pulsedial);
	free(answer);
	free(hangup);
	free(reset);
	free(retrn);
	free(escape);
}

void modem::ConfigureFromFile(const char * pathname)
{
	char ini[129], ton[129], pul[129], ans[129], han[129];
	char res[129], ret[129], esc[129], actualret[129];
	FILE * cfg = fopen(pathname, "r");
	if(cfg == NULL)
		fprintf(stderr, "modem error:  cannot open or use"
						" config file %s\n",	pathname);
	int n = fscanf(cfg, "%128[^\n]\n%128[^\n]\n%128[^\n]"
						"%128[^\n]\n%128[^\n]\n%128[^\n]"
						"%128[^\n]%128[^\n]",
						ini, ton, pul, ans, han, res, ret, esc);
	if(n != 8)
		fprintf(stderr, "Error in configuration file %s\n", pathname);
	else
	{
		char * p = ret;
		char * q = actualret;
		do 
		{
			if(*p == '\\')
			{
				++p;
				switch(*p)
				{
					case 'r': *p = '\r'; break;
					case 'n': *p = '\n'; break;
					case 't': *p = '\t'; break;
					case 'f': *p = '\f'; break;
					case 'v': *p = '\v'; break;
					case 'a': *p = '\a'; break;
					default: break;
				}
			}
			*q++ = *p;
		} while (*p++);
		free(init);
		free(tonedial);
		free(pulsedial);
		free(answer);
		free(hangup);
		free(reset);
		free(retrn);
		free(escape);

		init = strdup(ini);
		tonedial = strdup(ton);
		pulsedial = strdup(pul);
		answer = strdup(ans);
		hangup = strdup(han);
		reset = strdup(res);
		retrn = strdup(actualret);
		escape = strdup(esc);

		Initialize();
	}
	return;
}

void modem::Dial(char * number, boolean tone)
{
	SendString( (tone) ? tonedial : pulsedial);
	SendString(number);
	SendString(retrn);
}

void modem::Initialize()
{
	SendString(init);
	SendString(retrn);
}

void modem::Answer()
{
	SendString(answer);
	SendString(retrn);
}

void modem::Escape()
{
	SendString(escape);
	Pause(3000);
}

void modem::HangUp()
{
	SendString(hangup);
	SendString(retrn);
}

void modem::SendCommand(char * command)
{
	SendString(command);
	SendString(retrn);
}

void modem::Send(char ch) { SendChar(ch); }

void modem::Send(char * s) { SendString(s); }

void modem::Send(int i) { Write(&i, sizeof(int)); }

void modem::Send(long l) { Write(&l, sizeof(long)); }

void modem::Send(float f) { Write(&f, sizeof(float)); }

void modem::Send(double d) { Write(&d, sizeof(double)); }


// end of file modem.cpp

